#--
#
#################################################################################
# Copyright 2011 CENATIC y IEPALA
#
# Licencia con arreglo a la EUPL, Versión 1.1 o –en cuanto sean aprobadas por la Comisión Europea– 
# versiones posteriores de la EUPL (la «Licencia»);
# Solo podrá usarse esta obra si se respeta la Licencia.
# Puede obtenerse una copia de la Licencia en:
#
# http://www.osor.eu/eupl/european-union-public-licence-eupl-v.1.1
#
# Salvo cuando lo exija la legislación aplicable o se acuerde por escrito, 
# el programa distribuido con arreglo a la Licencia se distribuye «TAL CUAL»,
# SIN GARANTÍAS NI CONDICIONES DE NINGÚN TIPO, ni expresas ni implícitas.
# Véase la Licencia en el idioma concreto que rige los permisos y limitaciones que establece la Licencia.
#################################################################################
#
#++
# Controlador encargado de la gestión de los documentos.
# Este controlador es utilizado desde las secciones:
# * Sección proyectos: se utiliza para listar, comentar, subir y descargar documentos
# * Sección financiaciones: se utiliza para listar, comentar, subir y descargar documentos
class DocumentoController < ApplicationController

  # Solo elimina cuando el documento pertenece
  before_filter :autorizar, :only => [ :eliminar, :modificar_crear ]
  before_filter :espacio_seleccionado, :except => [ :index, :seleccionar_proyecto ]

  # Este metodo es solamente para tener una variable que utilizamos mucho en muchos metodos
  def espacio_seleccionado
    @espacio_id = session[:espacio_proyecto_seleccionado] if params[:seccion] == "proyectos"
    @espacio_id = session[:espacio_agente_seleccionado] if params[:seccion] == "agentes"
    @espacio_id = session[:espacio_seleccionado] if params[:seccion] == "documentos"
    @espacio_id = Espacio.find_by_nombre("Plantillas exportación").id  if params[:seccion] == "administracion"
  end


  # Permitimos modificar y borrar solo al propietario del documento y al coordinador
  def autorizar
    if params[:id]
      @documento = Documento.find(params[:id])
      if @documento.usuario_id && @documento.usuario_id != session[:usuario_identificado].id && ((@proyecto && @proyecto.usuario_x_proyecto.find_by_usuario_id( session[:usuario_identificado].id ).rol != "coordinador") || session[:usuario_identificado].administracion )
      render (:update) {|page|  page.mensaje_informacion params[:update], "No tiene permisos para borrar o modificar el documento.", :tipo_mensaje => "mensajefallo"}
      end
    end
  end


	# en proyectos y en financiación: se redirecciona por defecto a ordenado
  def index
    session[:espacio_seleccionado] = nil  if params[:seccion] == "documentos"
    session[:espacio_seleccionado] = Espacio.find_by_nombre("Plantillas exportación").id  if params[:seccion] == "administracion"
    session[:espacio_proyecto_seleccionado] = @proyecto.espacio.id if params[:seccion] == "proyectos"
    session[:espacio_agente_seleccionado] = @agente.espacio.id if params[:seccion] == "agentes"
    redirect_to :action => "ordenado"
  end

	# en proyectos y en financiación: establece los parametros de ordenación
  def ordenado
    session[:documento_asc_desc] = params[:asc_desc] ? params[:asc_desc] : "ASC" 
    session[:documento_orden] = params[:orden] ? params[:orden] : "adjunto_file_name" 
    redirect_to :action => "listado"
  end


  def seleccionar_espacio
    session[:espacio_seleccionado] = params[:id]  if params[:seccion] == "documentos"
    session[:espacio_proyecto_seleccionado] = params[:id] if params[:seccion] == "proyectos"
    session[:espacio_agente_seleccionado] = params[:id] if params[:seccion] == "agentes"
    session[:espacio_seleccionado] = Espacio.find_by_nombre("Plantillas exportación").id if params[:seccion] == "administracion"
    redirect_to :action => :listado
  end

  def listado
    #Si estamos en administracion forzamos siempre estar en el mismo espacio
    @espacio = Espacio.find(@espacio_id) if @espacio_id
    @documentos = @espacio.reload.documento if @espacio
    condiciones = ['espacio_padre_id = ?', @espacio.id ] if @espacio
    condiciones = 'espacio_padre_id is NULL and (definicion_espacio_proyecto is NULL or definicion_espacio_proyecto = "0") and 
                                                (definicion_espacio_agente is NULL or definicion_espacio_agente = "0") ' unless @espacio
    @espacios = Espacio.find :all, :conditions => condiciones
  end


#--
# ACCIONES COMUNES DE DOCUMENTOS: crear, editar, elminar, descargar
#++

	# en proyectos y en financiación: prepara el formulario de edición o creación
  def editar_nuevo
    if params[:documento]
      # Todo esto hay que hacerlo por el iframe de subida con ajax
      params[:tipo] ||= params[:url][:tipo] if params[:url]
      params[:objeto_id] ||= params[:url][:objeto_id] if params[:url]
      params[:i] ||= params[:url][:i] if params[:url]
      modificar_crear
    else
      @documento = params[:id] ?  Documento.find(params[:id]) : Documento.new
      carga_espacios_etiquetas
      espacio =  Espacio.find(@espacio_id)
      render :update do |page|
        # Si se estamos navegando por espacios no dejamos que se pueda tocar espacios de proyeto o espacios de plantillas
        if params[:menu] == "documentos_generales" and (espacio.definicion_espacio_proyecto_id or espacio.proyecto_id or 
          espacio.definicion_espacio_agente_id or espacio.agente_id or espacio.nombre == "Plantillas exportación" )
          page.mensaje_informacion params[:update], "No se puede incluir documentación para espacios de proyectos, o de agentes, o de administración, desde esta sección.", :tipo_mensaje => "mensajefallo"
        else
          page.formulario :partial => "formulario", :update => params[:update]  unless params[:pisa_formulario]
          page.replace 'formulariocontenedor', :partial => "formulario", :locals => { :update => params[:update], :update_listado => params[:update_listado] }  if params[:pisa_formulario]
        end
      end
    end
  end

  # Carga espacios y etiquetas
  def carga_espacios_etiquetas
    if params[:seccion] == "documentos"
      condiciones_espacio = "proyecto_id is NULL and nombre != 'Proyectos' and nombre != 'Plantillas exportación' 
                             and definicion_espacio_proyecto_id is NULL and definicion_espacio_proyecto is NULL " 
      @espacios = Espacio.find(:all, :conditions => condiciones_espacio).collect {|e| [e.nombre , e.id]}
    elsif params[:seccion] == "proyectos" or params[:seccion] == "agentes"
      @objeto = @proyecto ? @proyecto : @agente
      @espacios = @objeto.espacio.espacio_hijo
      @espacios = @espacios + [@objeto.espacio] - [Espacio.find(@proyecto ? session[:espacio_proyecto_seleccionado] : session[:espacio_agente_seleccionado])]
      @espacios = @espacios.collect {|e| [e.nombre , e.id]}
    end
    @etiquetas = @etiquetas_exportacion = Etiqueta.find(:all, :conditions => ["etiqueta.nombre LIKE ?", _("Exportacion") + " %"])
    @etiquetas = Etiqueta.find(:all, :conditions => "tipo = 'proyecto' or tipo ='comunes'") - @etiquetas_exportacion if params[:seccion] == "proyectos"
    @etiquetas = Etiqueta.find(:all, :conditions => {"tipo" => "comunes"}) - @etiquetas_exportacion if params[:seccion] == "documentos" or params[:seccion] == "agentes"
    @etiquetas_documento = @documento.etiqueta.collect{ |e| e.id }
  end



	# modifica o crea
  def modificar_crear
    @documento = Documento.find_by_id(params[:id]) || Documento.new
    # Solo ponemos el usuario_id si no existe o el documento es nuevo
    params[:documento][:usuario_id] = session[:usuario_identificado].id if !params[:id] || !@documento.usuario_id
    @documento.update_attributes params[:documento]
    if params[:tipo] && params[:tipo] != ""
      # Si estamos trabajando en las fuentes de verificacion, gastos, o transferencias vinculamos el documento a la fuente de verificacion
      #tipo, objeto_id = obtiene_tipo_y_objeto
      logger.info ">>>>>>>>>>>>>>> Asociamos Documento a un objeto " + params[:tipo] + " <<<<<<<<<<<<<<<<<<"
      @objeto = eval( params[:tipo] ).find_by_id(params[:objeto_id].to_i)
      @objeto.documento << @documento unless @objeto.nil? || @objeto.documento.find_by_id(@documento.id)
      #Cargamos los documentos del objeto para refrescar el listado
      @documentos = @objeto.documento
    else
      # Si estamos trabajando en documentos por espacio
      documento_x_espacio = DocumentoXEspacio.find_or_create_by_documento_id_and_espacio_id @documento.id, params[:documento_x_espacio][:espacio_id]
      cambio_de_espacio = false
      # Para cambiar un documento de espacio dentro de proyectos
      if params[:cambiar] and params[:cambiar][:espacio] == "1" and params[:cambiar][:espacio_id] != ""
        (DocumentoXEspacio.find_by_documento_id_and_espacio_id @documento.id, @espacio_id).destroy
        DocumentoXEspacio.create :documento_id => @documento.id, :espacio_id => params[:cambiar][:espacio_id]
        cambio_de_espacio = true if @espacio_id.to_s != params[:cambiar][:espacio_id] 
      end
    end
    #Finalmente guardamos las etiquetas vinculadas al documento
    @documento.etiqueta.clear
    @documento.etiqueta_ids = params["etiqueta"].to_a.collect {|p| p[0] if p[1] == "1"} if @documento.errors.empty?
    if @documento.errors.empty?
      responds_to_parent do
        # Si estamos en el listado de documentos
        unless params[:tipo] && params[:tipo] != ""
          i = params[:i] || params[:url][:i]
          render :update do |page|
            if cambio_de_espacio 
              page.remove "formularioinline"
              page.remove "formulariofondo"
              page.remove "formulariocontenedor"
              page.eliminar(:update => params[:update], :mensaje => { :errors => @documento.errors })
            elsif params[:id]
              page.modificar(:update => params[:update], :partial => "documento", :mensaje => { :errors => @documento.errors })
            else
              page.show "nuevos_documentos"
              page.modificar(:update => "documento_nuevo_" + params[:i], :partial => "nuevo_documento", :mensaje => { :errors => @documento.errors })
              page.replace "anadir", :inline => "<%= anadir(:url => {:action => (@proyecto ? 'asociar_o_nuevo' : 'editar_nuevo'), :i => (params[:i].to_i + 1).to_s, :update => 'formulario_anadir'}) %>"
            end
          end
        # Si es una vinculacion a un documento recargamos todo el listado
        else
          # Para otros objetos (en sublistado)
          render(:update){ |page|  page.modificar(:update_listado => params[:update_listado], :partial => "listado_asociados", :locals => {:objeto_id => params[:objeto_id], :tipo => params[:tipo] }, :mensaje => { :errors => @documento.errors}, :tipo_update => "sublistado") } 
        end
      end
    else
      # Si hay datos que fallan se vuelve al formulario
      carga_espacios_etiquetas
      responds_to_parent do
        render(:update){ |page|  page.recargar_formulario :partial => "formulario", :mensaje => { :errors => @documento.errors } }
      end
    end
  end    

	# en proyectos y en financiación: elimina documento
  def eliminar
    @documento = Documento.find(params[:id])
    @espacio =  Espacio.find(@espacio_id)
    # Si es un documento vinculado a un objeto
    if params[:menu] == "ejecucion"
      @objeto = eval( params[:tipo] ).find_by_id(params[:objeto_id])
      # desvinculamos el documento del objeto
      @objeto.documento.delete(@documento)
      # Si el documento no esta vinculado a ningun otro objeto lo borramos
      @documento.destroy if @documento.send(params[:tipo].tableize).empty?      
    # Si es un documento gestionado desde espacios de documento
    elsif params[:menu] == "documentos_proyecto" or params[:menu] == "documentos_agente"
      @espacio.documento.delete(@documento)
      # Si el documento no esta vinculado a ningun otro objeto lo borramos
      @documento.destroy if @documento.espacio.reload.empty? 
    elsif params[:menu] == "documentos_generales" or params[:menu] == "datos_configuracion"
      @documento.destroy if @documento.espacio.size < 2
      vinculado = @documento.espacio.reload.size < 2 ? false : true
    end
    render :update do |page|  
        page.eliminar(:update => params[:update], :mensaje => { :errors => @documento.errors, :eliminar => true }) unless vinculado
        page.mensaje_informacion(params[:update], "No se puede borrar el documento por que se encuentra vinculado a algun espacio de documentación de proyectos", :tipo_mensaje => "mensajefallo") if vinculado
    end
  end

  def descargar
    @documento = Documento.find(params[:id])
      # file = File.new(@document.document.path, "r")
      # send_data(file.read,
      #  :filename     =>  @document.document_file_name,
      #  :type         =>  @document.document_content_type,
      #  :disposition  =>  'inline') 

    # Antes de descargar, comprobar que exista el fichero y marcar el error si no esta
    if File.exists?(@documento.adjunto.path)       
      send_file @documento.adjunto.path, :type => @documento.adjunto_content_type, :disposition => 'inline'
    else
      flash[:error] = _("No se pudo encontrar el documento. Contacte con el administrador del sistema.")
      redirect_to :action => 'listado'
    end
  end


   # muestra los documentos vinculados a un objeto
  def listado_asociados
    @documentos = []
    @objeto = eval( params[:tipo] ).find_by_id(params[:objeto_id])
    @documentos = @objeto.documento if @objeto
    render :update do |page|
      page.replace_html params[:update], :partial => "listado_asociados", :locals => { :objeto_id => params[:objeto_id], :tipo => params[:tipo], :documentos => @documentos, :update_listado => params[:update]  }
    end
  end

  def asociar_o_nuevo
    espacio = Espacio.find(@espacio_id)
    render :update do |page|
      page.formulario :partial => "formulario_asociar_o_nuevo", :tipo => params[:tipo], :objeto_id => params[:objeto_id], :update => params[:update] 
    end
  end

  def asociar_documentos
    @objeto = @proyecto ? @proyecto : @gasto
    if params[:tipo] == "FuenteVerificacion"
      #Si estamos asociando a una fuente de verificacion
      documentos_fuente =  FuenteVerificacion.find(params[:objeto_id]).documento
      documentos = (FuenteVerificacionXDocumento.find(:all, :include => [:documento], :conditions => {"documento.proyecto_id" => 2}).collect {|d| d.documento}).uniq
      documentos = documentos - documentos_fuente
    elsif params[:tipo] == "Gasto"
      # Si estamos asociando a un gasto
      gasto = Gasto.find params[:objeto_id]
      gastos = Gasto.find_all_by_agente_id(@agente.id) if params[:seccion] == "agentes"
      gastos = Gasto.find(:all, :include => [:gasto_x_proyecto], :conditions => {"gasto_x_proyecto.proyecto_id" => @proyecto.id}) if params[:seccion] == "proyectos"
      documentos = (gastos.inject(Array.new){|sum, n| sum + n.documento} - gasto.documento).uniq
    else 
      # Si estamos asociando dentro del espacio de proyectos
      documentos = Documento.find_all_by_proyecto_id_and_agente_id(nil, nil)
      plantillas = (Espacio.find_by_nombre "Plantillas exportación").documento
      documentos  = documentos - plantillas
    end
      @documentos = documentos.collect {|d| [d.adjunto_file_name, d.id]}
    render(:update) { |page| page.replace 'formulariocontenedor', :partial => "formulario_asociar", :locals => { :update => params[:update] } }
  end

  #Asociamos un documento a un objeto o a un espacio
  def asociar
    if params[:tipo] and params[:tipo] != ""
      # Si estamos asociando a un objeto asociamos el documento al objeto
      @objeto = eval( params[:tipo] ).find_by_id(params[:objeto_id])
      @relacion = eval(params[:tipo] + "XDocumento").create( params[:tipo].tableize + "_id" => params[:objeto_id], "documento_id" => params[:documento][:id] )
      @documentos = @objeto.documento if @objeto
    elsif
      # Si no hay objeto asociamos el documento al espacio de navegacion
      @documento = Documento.find(params[:documento][:id])
      espacio_seleccionado_id = session[:espacio_proyecto_seleccionado] if @proyecto 
      espacio_seleccionado_id = session[:espacio_agente_seleccionado] if @agente
      espacio_seleccionado_id = session[:espacio_seleccionado] unless @proyecto or @agente
      @relacion = DocumentoXEspacio.create(:espacio_id => espacio_seleccionado_id, :documento_id => @documento.id )
    end
    if @relacion.errors.empty?
      if params[:tipo]    
        # Si la asociacion es a un objeto refrescamos el listado de documentos del objeto
        render(:update){ |page|  page.modificar(:update_listado => params[:update_listado], :partial => "listado_asociados", :locals => {:objeto_id => params[:objeto_id], :tipo => params[:tipo]}, :mensaje => { :errors => @relacion.errors}, :tipo_update => "sublistado") }
      elsif
        # Si el documento se asocia a un espacio lo mostramos en la zona de nuevos documentos
        render :update do |page|
          page.show "nuevos_documentos"
          page.modificar(:update => "documento_nuevo_" + params[:i], :partial => "nuevo_documento", :mensaje => { :errors => @documento.errors })
          page.replace "anadir", :inline => "<%= anadir(:url => {:action => 'asociar_o_nuevo', :i => (params[:i].to_i + 1).to_s, :update => 'formulario_anadir'}) %>"
        end
      end
    else
      # Si hay datos que fallan se vuelve al formulario
      @documentos = @proyecto.documento.collect {|d| [d.adjunto_file_name, d.id]}
      render(:update){ |page|  page.recargar_formulario :partial => "formulario_asociar", :mensaje => { :errors => @relacion.errors } }
    end
  end



end
