#--
#
#################################################################################
# Copyright 2011 CENATIC y IEPALA
#
# Licencia con arreglo a la EUPL, Versión 1.1 o –en cuanto sean aprobadas por la Comisión Europea– 
# versiones posteriores de la EUPL (la «Licencia»);
# Solo podrá usarse esta obra si se respeta la Licencia.
# Puede obtenerse una copia de la Licencia en:
#
# http://www.osor.eu/eupl/european-union-public-licence-eupl-v.1.1
#
# Salvo cuando lo exija la legislación aplicable o se acuerde por escrito, 
# el programa distribuido con arreglo a la Licencia se distribuye «TAL CUAL»,
# SIN GARANTÍAS NI CONDICIONES DE NINGÚN TIPO, ni expresas ni implícitas.
# Véase la Licencia en el idioma concreto que rige los permisos y limitaciones que establece la Licencia.
#################################################################################
#
#++
# Controlador encargado de la gestion de la entidad Libro 
#
# NOTA: La terminologia utilizada en las vistas es cuenta
# Controlador encargado de la gestión de libros. Este controlador es utilizado desde las secciones:
# * Sección administración: se utiliza para crear libros y asignarles usuarios.
class LibroController < ApplicationController
  
  # en administracion: se redirecciona por defecto a ordenado
  def index
    redirect_to :action => 'ordenado'
  end

  # en administracion: lista las agentes que hay en el sistema
  def listado
    @listado_monedas = Moneda.find(:all, :order => "nombre").collect {|p| [p.nombre , p.id]}
    @listado_agentes = Agente.find(:all, :order => "nombre").collect {|p| [p.nombre, p.id]}
    @listado_paises = Pais.find(:all, :order => "nombre").collect {|p| [p.nombre, p.id]}
    #@libro = params[:id] ?  Libro.find(params[:id]) : nil
    # @libros = @paginado = Libro.paginate_by_agente_id( session[:libro_agente], :order => "nombre", :page => params[:page], :per_page => session[:por_pagina] or 20 )
    @libros = @paginado = Libro.paginate( :all, :order => (session[:libro_orden] + " " + session[:libro_asc_desc]) , :include => [ :pais, :agente ], :page => params[:page], :per_page => (session[:por_pagina] or 20) )
  end

  # en administracion: establece los parametros de ordenación
  def ordenado
    session[:libro_asc_desc] = params[:asc_desc] ? params[:asc_desc] : "ASC"   
    session[:libro_orden] = params[:orden] ? params[:orden] : "nombre" 
    redirect_to :action => "listado"
  end
  
  # en administracion: prepara el formulario de edición o creación
  def editar_nuevo
    @libro = Libro.find_by_id(params[:id]) || Libro.new
    @listado_monedas = Moneda.find(:all, :order => "nombre").collect {|p| [p.nombre , p.id]}
    @listado_agentes = Agente.find(:all, :conditions => {:implementador => true}, :order => "nombre").collect {|p| [p.nombre, p.id]}
    @listado_paises = Pais.find(:all, :order => "nombre").collect {|p| [p.nombre, p.id]}
    @paises = Pais.find(:all).collect {|p| [p.nombre, p.id]}
    render :partial => "formulario"
  end

  # en administracion: modifica o crea
  def modificar_crear
    @libro = Libro.find_by_id(params[:id]) || Libro.new
    @libro.update_attributes params[:libro]
    flash[:error] = @libro
    redirect_to :action => "listado"
  end

  # en administracion: elimina
  def eliminar
    @libro = Libro.find(params[:id])
    @libro.destroy
    flash[:error_eliminar] = @libro
    redirect_to :action => 'listado'
  end


  # en administracion: muestra el formulario para asociar usuarios
  def usuarios
    #sleep 10
    @libro = @objeto = Libro.find(params[:id])
    # @libros = @libro.usuario_x_libro
    @usuarios = @libro.usuario_x_libro
    @listado_usuarios = Usuario.find(:all).collect {|a| [a.nombre, a.id]}
    render :partial => "usuario/usuarios"
  end

  # en administracion: guarda la asociación de usuarios
  def guardar_usuarios
    @libro = @objeto = Libro.find(params[:id])
    @libro.actualizar_usuario_x_libro params[:usuarios]
    render  :template => "usuario/guardar_usuarios"
  end

  # en administracion: añade un usuario al formulario
  def anadir_usuario
    @objeto = Libro.find(params[:id])
    @listado_usuarios = Usuario.find(:all).collect {|a| [a.nombre, a.id]}
    render :template => "usuario/anadir_usuario" 
  end

end
