#--
#
#################################################################################
# Copyright 2011 CENATIC y IEPALA
#
# Licencia con arreglo a la EUPL, Versión 1.1 o –en cuanto sean aprobadas por la Comisión Europea– 
# versiones posteriores de la EUPL (la «Licencia»);
# Solo podrá usarse esta obra si se respeta la Licencia.
# Puede obtenerse una copia de la Licencia en:
#
# http://www.osor.eu/eupl/european-union-public-licence-eupl-v.1.1
#
# Salvo cuando lo exija la legislación aplicable o se acuerde por escrito, 
# el programa distribuido con arreglo a la Licencia se distribuye «TAL CUAL»,
# SIN GARANTÍAS NI CONDICIONES DE NINGÚN TIPO, ni expresas ni implícitas.
# Véase la Licencia en el idioma concreto que rige los permisos y limitaciones que establece la Licencia.
#################################################################################
#
#++
# Controlador encargado de las vistas resumen financieras para una agente implementador. Este controlador es utilizado desde las secciones agentes.

class ResumenAgenteController < ApplicationController

  helper :resumen_proyecto
  
  # en agente: se redirecciona por defecto a presupuesto
  def index
      if @agente.etapa.empty? 
        flash[:error] = _("Tiene que definir por lo menos una etapa")
        redirect_to :controller => :datos_agente, :action => :etapas
      else
        redirect_to :action => :presupuesto
      end
  end

  def gasto
     @listado_moneda = @agente.moneda.collect{|e| [e.nombre, e.id]} + [["Todas las monedas (con tasa aplicada)","todas"]]
    # Hacemos la busqueda si se seleccionan los criterios
    estado_gasto if params[:moneda]
  end

  def estado_gasto
    @fecha_de_inicio = fecha params["fecha_inicio"]
    @fecha_de_fin = (fecha(params["fecha_fin"]) >> 1) -1 # Lo adelantamos un mes para que sea mas coherente

    # Recogemos la tabla de resumen general
    condiciones = {"ocultar_agente" => false} unless params[:mostrar_partidas_ocultas] == "1"
    partidas = Partida.find(:all, :conditions => condiciones, :order => "codigo")
    filas = partidas.collect{|i| {"id" => i.id, "nombre" => i.codigo_nombre}}
    columnas = [{"id" => "1", "nombre" => "Presupuesto"}, {"id" => "2", "nombre" => "Gastos"}]
#    presupuestos = VPresupuestoDetallado.partida_de_implementador_sin_proyecto(params[:moneda], @agente.id, @fecha_de_inicio, @fecha_de_fin , "0").collect {|p| {"columna_id" => "1", "importe" => p.importe , "fila_id" => p.fila_id }}
    presupuestos = VPresupuestoAgenteDetallado.partida(@agente.id, @fecha_de_inicio, @fecha_de_fin , params[:moneda],  params[:tasa_cambio], nil).collect {|p| {"columna_id" => "1", "importe" => p.importe , "fila_id" => p.partida_id.to_s }}
    gastos = VGastoAgente.partida(@agente.id, @fecha_de_inicio, @fecha_de_fin, params[:moneda], params[:tasa_cambio], nil).collect {|p| {"columna_id" => "2", "importe" => p.importe , "fila_id" => p.partida_id.to_s }}
    datos = presupuestos + gastos
    titulo_datos =[ "Moneda: #{ params[:moneda] == 'todas' ?  'Todas las monedas' : Moneda.find(params[:moneda]).abreviatura} " ,
               "Tasa cambio: #{(params[:tasa_cambio] == '1' or params[:moneda] == 'todas') ? 'Aplicada': 'No aplicada'}" ,
               "Fechas: #{  @fecha_de_inicio.to_s + '/' + @fecha_de_fin.to_s } " ]

    titulo_resumen =  ["Resumen de gasto comparado con el presupuesto."] + titulo_datos
    tabla_resumen = { :titulo => titulo_resumen, :filas => filas, :columnas => columnas, :datos => datos, :otros => { :fila_suma => true, :columna_resta => true, :columna_desviacion => true } }

     # Comprobamos que no hay partidas ocultas con datos de agente para comunicarlo al usuario
     presupuestos.each do |p|
       flash.now[:error] = "Existen partidas ocultas que contienen presupuestos. Seleccione mostrar partidas ocultas para verlas"  if (filas.select{|f| f["id"].to_s == p["fila_id"].to_s}).empty?
     end
     gastos.each do |p|
       flash.now[:error] = "Existen partidas ocultas que contienen gastos. Seleccione mostrar partidas ocultas para verlas"  if (filas.select{|f| f["id"].to_s == p["fila_id"].to_s}).empty?
     end

    # Mostramos el detalle del seguimiento por subpartidas del agente
     @titulo_subpartida = ["Detalle del presupuesto frente al gasto del agente por subpartidas"] + titulo_datos
     @subpartidas =  [:cabecera => [[ "Nombre" , "2" ], ["Presupuesto" ,"1_2_td"], [ "Gasto"  ,"1_2_td"], ["Diferencia"  ,"1_2_td"],  ["Desviacion"  ,"1_2_td"]] ]
      for partida in partidas
          importe_presupuesto = VPresupuestoAgenteDetallado.partida(@agente.id, @fecha_de_inicio, @fecha_de_fin, params[:moneda] , params[:tasa_cambio], partida.id.to_s ).first
          importe_gasto = VGastoAgente.partida(@agente.id, @fecha_de_inicio, @fecha_de_fin, params[:moneda] , params[:tasa_cambio], partida.id.to_s ).first
          gasto = importe_gasto ? importe_gasto[:importe] : 0
          presupuesto = importe_presupuesto ? importe_presupuesto[:importe] : 0
          @subpartidas.push(:cabecera => [[ partida.codigo_nombre , "2" ], [presupuesto ,"1_2_td"], [ gasto  ,"1_2_td"], [(presupuesto - gasto)  ,"1_2_td"], 
                                          [ (presupuesto == 0 ? "-" : (presupuesto - gasto)/presupuesto * 100)  ,"1_2_td"]])
          for subpartida in   Subpartida.find_all_by_agente_id_and_partida_id( @agente.id, partida.id)
            pres = VPresupuestoAgenteDetallado.subpartida(@agente.id, @fecha_de_inicio, @fecha_de_fin ,params[:moneda], params[:tasa_cambio], partida.id.to_s, subpartida.id.to_s, nil)
            importe_gasto = VGastoAgente.subpartida(@agente.id, @fecha_de_inicio, @fecha_de_fin, params[:moneda] , params[:tasa_cambio], partida.id.to_s, subpartida.id.to_s, nil)
            gasto = importe_gasto.empty? ? 0 : importe_gasto.first[:importe]
            presupuesto = pres.empty? ? 0 : pres.first[:importe]
            @subpartidas.push(:contenido => [subpartida.nombre, presupuesto, gasto, (presupuesto - gasto), (presupuesto == 0 ? "-" : (presupuesto - gasto)/presupuesto * 100)])  
          end
            importe_pre = VPresupuestoAgenteDetallado.subpartida(@agente.id, @fecha_de_inicio, @fecha_de_fin ,params[:moneda], params[:tasa_cambio], partida.id.to_s, nil, true)
            importe_gasto = VGastoAgente.subpartida(@agente.id, @fecha_de_inicio, @fecha_de_fin, params[:moneda] , params[:tasa_cambio], partida.id.to_s, nil, true)
            gasto = importe_gasto.empty? ? 0 : importe_gasto.first[:importe]
            pres = importe_pre.empty? ? 0 : importe_pre.first[:importe]
            @subpartidas.push(:contenido => ["Sin subpartida", pres, gasto, (pres - gasto), (pres == 0 ? "-" : (pres - gasto)/pres * 100)])  unless gasto == 0 and pres == 0
      end
   

    @tablas =[tabla_resumen]

    # Recogemos todo en una tabla común
    respond_to do |format|
      format.xls do
        @resumen = [	{:tabla => tabla_resumen},
			{:objeto => {	:tipo => "v_presupuesto_subpartida",
					:nombre => "Presupuesto por Subpartidas",
					:titulo => @titulo_presupuesto_subpartida,
					:objetos => @v_presupuesto_subpartida } },
			{:objeto => {	:tipo => "v_gasto_subpartida",
					:nombre => "Gasto por Subpartidas",
					:titulo => @titulo_gasto_subpartida,
					:objetos => @v_gasto_subpartida } }  ]
        render 'comunes_xls/resumen', :layout => false
      end if params[:fichero] == "1"
      format.html if params[:fichero] == "0"
    end
  end

  def presupuesto
    @listado_moneda = (@agente.moneda.collect{|e| [e.nombre, e.id]}) + [["Todas las monedas (T.Cambio aplicada)", "todas"]]
    @listado_etapas = @agente.etapa.collect{|e| [e.nombre, e.id]}
    estado_presupuesto if params[:moneda] 
  end

  def estado_presupuesto
   condiciones = {"ocultar_agente" => false} unless params[:mostrar_partidas_ocultas] == "1"
   partidas = Partida.find(:all, :conditions => condiciones, :order => "codigo")
   # Recogemos los datos de la tabla primera resumen del presupuesto de agente frente al presupuesto de proyecto
   #filas =  partidas.collect {|i| {"id" => i.id, "nombre" => i.codigo_nombre, "url" => {:action => "subpartida", :etapa =>  params[:etapa], :moneda => params[:moneda], :tasa_cambio => ((params[:tasa_cambio] == "1" or params[:moneda] == "todas") ? "1": "0" ), :partida => i.id}, "update" => "subpartida"}} if params[:detallado] == "0"
   filas =  partidas.collect {|i| {"id" => i.id, "nombre" => i.codigo_nombre} }  # if params[:detallado] == "1"       
   columnas = [{"id" => "2", "nombre" => "Pto. Proyectos"}, {"id" => "1", "nombre" => "Pto. Agente"}]
   etapa = Etapa.find params[:etapa]
   presupuestos_proyectos = VPresupuestoDetallado.partida_de_implementador_sin_proyecto(params[:moneda], @agente.id, etapa.fecha_inicio, etapa.fecha_fin , params[:tasa_cambio], nil, true).collect {|p| {"columna_id" => "2", "importe" => p.importe , "fila_id" => p.fila_id }}
   presupuestos_delegacion = VPresupuestoAgente.partida_implementador(@agente.id, etapa.id, params[:moneda], params[:tasa_cambio], nil).collect {|p| {"columna_id" => "1", "importe" => p.importe , "fila_id" => p.fila_id }}
   datos = presupuestos_proyectos + presupuestos_delegacion
   titulo =[ "Moneda: #{ params[:moneda] == 'todas' ?  'Todas las monedas' : Moneda.find(params[:moneda]).abreviatura} " ,
             "Tasa cambio: #{(params[:tasa_cambio] == '1' or params[:moneda] == 'todas') ? 'Aplicada': 'No aplicada'}" ,
             "Etapa: #{ etapa.nombre + ':' + etapa.fecha_inicio.to_s + '/' + etapa.fecha_fin.to_s } " ]
   titulo_resumen = ["Resumen del presupuesto de proyectos (ingresos) y de agente (gastos)"] + titulo
   tabla = { :titulo => titulo_resumen, :filas => filas, :columnas => columnas, :datos => datos, :otros => { :fila_suma => true, :columna_resta => true } }

   # Comprobamos que no hay partidas ocultas con datos de agente
   presupuestos_proyectos.each do |p|
     flash.now[:error] = "Existen partidas ocultas que contienen presupuestos de proyecto. Seleccione mostrar partidas ocultas para verlas"  if (filas.select{|f| f["id"].to_s == p["fila_id"].to_s}).empty?
   end
   presupuestos_delegacion.each do |p|
     flash.now[:error] = "Existen partidas ocultas que contienen presupuestos de agente. Seleccione mostrar partidas ocultas para verlas"  if (filas.select{|f| f["id"].to_s == p["fila_id"].to_s}).empty?
   end

   #Recogemos la tabla del detalle del presupuesto del agente por subpartidas y la de detalle de presupuesto por proyectos   
   @subpartidas = Array.new
   @presupuesto_proyectos = Array.new
   for partida in partidas
      #Buscamos el valor por partidas del presupuesto por agente
      importe_partida = VPresupuestoAgente.partida_implementador(@agente.id, etapa.id, params[:moneda], params[:tasa_cambio], partida.id).first
      @subpartidas.push(:cabecera => [[ partida.codigo_nombre , "2" ], [(importe_partida ? importe_partida[:importe] : "0,00") ,"1_2_td"]])
    
      for subpartida in  Subpartida.find_all_by_agente_id_and_partida_id( @agente.id, partida.id)
        valor = VPresupuestoAgente.subpartida_partida(@agente.id, params[:etapa], params[:moneda], params[:tasa_cambio], partida.id.to_s, (subpartida ? subpartida.id.to_s : nil))
        @subpartidas.push(:contenido => [(subpartida ? subpartida.nombre : "sin subpartida"), valor.first[:importe]]) unless valor.nil? or valor.empty?
      end
      #buscamos el valor por partidas del presupuesto por proyecto
      partida_proyectos = VPresupuestoDetallado.partida_de_implementador_sin_proyecto(params[:moneda], @agente.id, etapa.fecha_inicio, etapa.fecha_fin , params[:tasa_cambio], partida.id, true).first
      @presupuesto_proyectos.push(:cabecera => [[ partida.codigo_nombre , "2" ], ["&nbsp;","2"], [(partida_proyectos  ? partida_proyectos[:importe] : "0,00") ,"1_2_td"]]) 
      detalle_partida_proyecto = VPresupuestoDetallado.implementador_sin_proyecto(params[:moneda], @agente.id, etapa.fecha_inicio, etapa.fecha_fin , params[:tasa_cambio], partida.id, true)
      for detalle in detalle_partida_proyecto
         @presupuesto_proyectos.push(:contenido => [detalle.proyecto_nombre,  detalle.presupuesto_concepto,(detalle.importe || "0,00") ]) 
      end
    end
     @titulo_presupuesto_subpartida = ["Detalle del presupuesto del agente (gastos) por subpartidas"] + titulo   
     @titulo_presupuesto_proyectos = ["Detalle del presupuesto por proyectos (ingresos)"] + titulo
     # Recogemos todo en una tabla común
     @tablas = [tabla]
     respond_to do |format|
        format.xls do
          @nombre = "Resumen Presupuesto" + @agente.nombre
          render 'comunes_xls/resumen_tabla', :layout => false
        end if params[:fichero] == "1"
        format.html if params[:fichero] == "0" 
      end
  end

  # Ya no se utiliza!
  def subpartida
   @columnas_agente = [{"id" => "1", "nombre" => "Importe"}]
   @filas_agente = Presupuesto.find(:all, :select => "subpartida.nombre", :include => [:subpartida], :conditions => {"agente_id" => @agente.id, "partida_id" => params[:partida], :proyecto_id => nil} , :group => "subpartida.nombre" ).collect{ |p| ({"nombre" => p.subpartida.nombre, "id" =>  p.subpartida.id }) if p.subpartida }
   @etapa = Etapa.find(params[:etapa])
   @datos_agente = VPresupuestoAgente.subpartidas_partida(@agente.id, params[:etapa], params[:moneda], params[:tasa_cambio], params[:partida]).collect{|p| {"columna_id" => "1", "importe" => p.importe , "fila_id" => p.fila_id }}

   @columnas_proyecto = [{"id" => "1", "nombre" => "Importe"} ]
   presupuesto_proyecto = VPresupuestoDetallado.implementador_sin_proyecto(params[:moneda], @agente.id, @etapa.fecha_inicio, @etapa.fecha_fin , params[:tasa_cambio], params[:partida])
   @filas_proyecto = presupuesto_proyecto.collect {|p| {"nombre" => p.proyecto_nombre , "id" => p.presupuesto_id}}
   @datos_proyecto  = presupuesto_proyecto.collect {|p| {"columna_id" => "1" , "fila_id" => p.presupuesto_id.to_s, "importe" => p.importe}}
  
   render :partial => "subpartida" 
  end

	# en agentes: selecciona la caja para sacar el informe de arqueo
  def arqueo_caja
    @listado_libros = eval( "@" + singularizar_seccion ).libro.select{|l| session[:usuario_identificado].libro.include? l}.collect {|a|[a.nombre, a.id]}
    @listado_etapa = @agente.etapa.collect{|e| [e.nombre, e.id]}

    # Si hemos especificado un libro para ver el informe
    if params[:selector] and params[:selector][:libro] != ''
      @libro = Libro.find(params[:selector][:libro])
      @moneda = @libro.moneda
      @etapa = Etapa.find_by_id(params[:selector][:etapa])

      fecha_inicio = @etapa ? @etapa.fecha_inicio : nil
      fecha_fin = @etapa ? @etapa.fecha_fin : nil

      arqueo = @libro.arqueo [], fecha_inicio, fecha_fin

      # Prepara la salida de datos
      lineas = Array.new
      lineas.push( :cabecera => [ [_("Fecha"),"1_2"], [_("Tipo"),"1_2"], [_("Concepto"),"3_2"], [_("Entradas"),"1_2_td"], [_("Salidas"),"1_2_td"] ])
      lineas += arqueo[:filas]
      lineas.push( :contenido => [ '','','','','' ] )
      lineas.push( :cabecera => [ [_("Totales"),"1_2"], ["","1_2"], ["","3_2"], [arqueo[:entrante],"1_2_td"], [arqueo[:saliente],"1_2_td"], [arqueo[:entrante] - arqueo[:saliente],"1_2_td"] ])
      arqueo[:totales].each do |k,v|
        lineas.push( :contenido => [ '',k,'',v["Entrante"]||'',v["Saliente"]||'','' ] )
      end

      @resumen = Array.new
      nombre = "arqueo_caja"
      titulo = "Arqueo Cuenta/Caja " + @libro.nombre + " (" + @libro.moneda.nombre + ") / Etapa: " +
                        (@etapa ? @etapa.nombre + " (desde " + @etapa.fecha_inicio.strftime('%d/%m/%Y') + " hasta " + @etapa.fecha_fin.strftime('%d/%m/%Y') + ")" : "Todas")
      @resumen.push(:listado => {:nombre => nombre, :titulo => titulo, :lineas => lineas})

      respond_to do |format|
        format.html do
          render :template => 'comunes/arqueo_caja', :layout => (params[:sin_layout] ? false : true)
        end unless params[:selector] && params[:selector][:fichero] == '1'
        format.xls do
          render 'comunes_xls/resumen', :layout => false
        end if params[:selector] && params[:selector][:fichero] == '1'
      end
    else
      render :template => "comunes/arqueo_caja", :layout => (params[:sin_layout] ? false : true)
    end
 
  end

end
