  #--
#
#################################################################################
# Copyright 2011 CENATIC y IEPALA
#
# Licencia con arreglo a la EUPL, Versión 1.1 o –en cuanto sean aprobadas por la Comisión Europea– 
# versiones posteriores de la EUPL (la «Licencia»);
# Solo podrá usarse esta obra si se respeta la Licencia.
# Puede obtenerse una copia de la Licencia en:
#
# http://www.osor.eu/eupl/european-union-public-licence-eupl-v.1.1
#
# Salvo cuando lo exija la legislación aplicable o se acuerde por escrito, 
# el programa distribuido con arreglo a la Licencia se distribuye «TAL CUAL»,
# SIN GARANTÍAS NI CONDICIONES DE NINGÚN TIPO, ni expresas ni implícitas.
# Véase la Licencia en el idioma concreto que rige los permisos y limitaciones que establece la Licencia.
#################################################################################
#
#++
class Espacio < ActiveRecord::Base

  untranslate_all  
  belongs_to :espacio_padre, :class_name => 'Espacio' , :foreign_key => 'espacio_padre_id'
  has_many :espacio_hijo, :class_name => 'Espacio' , :foreign_key => 'espacio_padre_id'
  has_many :espacios_definidos, :class_name => 'Espacio' , :foreign_key => 'definicion_espacio_proyecto_id'
  belongs_to :definicion_espacio, :class_name => 'Espacio' , :foreign_key => 'definicion_espacio_proyecto_id'
  has_many :espacios_definidos_agente, :class_name => 'Espacio' , :foreign_key => 'definicion_espacio_agente_id'
  belongs_to :definicion_agente, :class_name => 'Espacio' , :foreign_key => 'definicion_espacio_agente_id'
  #has_many :documento
  has_many :documento_x_espacio, :dependent => :destroy
  has_many :documento, :through => :documento_x_espacio
  belongs_to :proyecto
  belongs_to :agente

  #validate :nombre_mayusculas 
  validates_presence_of :nombre, :message => _("Nombre") + _(" no puede estar vacío.")
  validates_uniqueness_of :nombre, :scope => [:espacio_padre_id, :definicion_espacio_proyecto, :definicion_espacio_agente], 
                          :message => _("Nombre") + _(" repetido.")
  before_validation_on_update :comprobar_estado_padre
  before_destroy :espacio_raiz_de_agente, :espacio_raiz_de_proyecto, :espacio_vacio
  after_destroy :borra_espacios_de_agente, :borra_espacios_de_proyecto
  after_save :modificar_crear_espacios_de_agente, :modificar_crear_espacios_de_proyecto

  # Si es un espacio de definicion_espacio_proyecto creamos una espacio asociado para cada espacio de proyecto
  def modificar_crear_espacios_de_proyecto
    for proyecto in  Proyecto.find :all, :conditions => {:convenio_accion => nil}
      espacio = Espacio.find_or_create_by_espacio_padre_id_and_definicion_espacio_proyecto_id proyecto.espacio.id, id if proyecto.espacio
      espacio.update_attributes :nombre => nombre, :gasto => gasto, :fuente_verificacion => fuente_verificacion, :transferencia => transferencia if proyecto.espacio
      espacio.errors.each {|e, m|  errors.add "", m }
    end if respond_to?('definicion_espacio_proyecto') && definicion_espacio_proyecto 
  end

  # Si es un espacio de definicion_espacio_agente creamos una espacio asociado para cada espacio de agente
  def modificar_crear_espacios_de_agente
    for espacio_agente in  Espacio.find(:all, :conditions => "agente_id is not NULL")
      espacio = Espacio.find_or_create_by_espacio_padre_id_and_definicion_espacio_agente_id_and_nombre espacio_agente.id, id, nombre
      espacio.update_attributes :nombre => nombre, :gasto => gasto, :transferencia => transferencia
      espacio.errors.each {|e, m|  errors.add "", m }
    end if respond_to?('definicion_espacio_agente') && definicion_espacio_agente
  end


  # Si es un espacio de definicion_espacio_proyecto eliminamos un espacio igual por cada proyecto. Siempre y cuando no contenga documentos.
  def borra_espacios_de_proyecto
    for espacio in  espacios_definidos
      espacio.destroy
      espacio.errors.each {|e, m|  errors.add "", m }
    end if definicion_espacio_proyecto
  end
 
  # Si es un espacio de definicion_espacio_agente eliminamos un espacio igual por cada agente. Siempre y cuando no contenga documentos.
  def borra_espacios_de_agente
    for espacio in  espacios_definidos_agente
      espacio.destroy
      espacio.errors.each {|e, m|  errors.add "", m }
    end if definicion_espacio_agente
  end

  # Comprobaciones previas al borrado 
  def espacio_raiz_de_proyecto
    errors.add("", "Espacio asociado a proyecto. No se puede borrar") if proyecto_id
    return  (proyecto_id ? false : true)
  end

  # Comprobaciones previas al borrado 
  def espacio_raiz_de_agente
    errors.add("", "Espacio asociado a agente. No se puede borrar") if agente_id
    return  (agente_id ? false : true)
  end

  # Comprobaciones previas al borrado  
  def espacio_vacio
    errors.add("", "Este espacio contiene documentos no se puede eliminar") unless documento.empty?
    return  documento.empty? 
  end

  def nombre_mayusculas
    self.nombre = self.nombre.upcase
  end

  def directory?
    true
  end

  def ruta
      (((espacio_padre.ruta || "") + " / " + espacio_padre.nombre) if espacio_padre) || ""
  end
  
  def comprobar_estado_padre
    errors.add( "estado_superior_id", _("No puede tener como espacio 'superior' el espacio seleccionado"))  if (espacio_padre.ruta.include?(Espacio.find(id).nombre) or espacio_padre_id == id) if espacio_padre
    return false unless errors.empty?
  end
  

end
