class ChangeTransferencia < ActiveRecord::Migration
  def self.up
    errores = false
    # Generamos los nuevos campos necesarios
    add_column :transferencia, :fecha_enviado, :date
    add_column :transferencia, :importe_enviado, :decimal, :precision => 24, :scale => 2
    add_column :transferencia, :libro_origen_id, :integer
    add_column :transferencia, :fecha_recibido, :date
    add_column :transferencia, :importe_recibido, :decimal, :precision => 24, :scale => 2
    add_column :transferencia, :importe_cambiado, :decimal, :precision => 24, :scale => 2
    add_column :transferencia, :libro_destino_id, :integer
    Transferencia.reset_column_information

    # Agrupamos las transferencias vinculadas
    # Primero las entrantes (con o sin vinculada, borrando las vinculadas salientes)
    Transferencia.all(:conditions => {:entrante_saliente => 'entrante'}).each do |transf|
      # Obtenemos la vinculada
      transf_vinculada = Transferencia.find_by_id(transf.transferencia_id)
      # Corregimos el tipo "cheque" (eliminado)
      transf.tipo = "retirada" if transf.tipo == "cheque"
      # Rellenamos los datos segun la entrante
      transf.fecha_recibido = transf.fecha
      # Adivina importe recibido segun la tasa de cambio
      transf.importe_recibido = transf.importe * transf.tasa_cambio
      transf.importe_cambiado = transf.importe
      transf.libro_destino_id = transf.libro_id
      transf.libro_origen_id = transf.libro_receptor_emisor_id
      # Y rellenamos los datos segun la vinculada (saliente)
      if transf_vinculada
        transf.fecha_enviado = transf_vinculada.fecha
        # Corrige la fecha si se envio despues de recibirse
        if transf.fecha_enviado > transf.fecha_recibido
          puts "--------> Ajustando: La fecha de envio es posterior a la de recepcion (" + transf.fecha_enviado.to_s + " > " + transf.fecha_recibido.to_s + ") para la transferencia con ID: " + transf.id.to_s
          transf.fecha_recibido = transf.fecha_enviado
        end
        transf.importe_enviado = transf_vinculada.importe
        # Ajusta importe recibido al real
        transf.importe_recibido = transf_vinculada.importe
        # Ajusta la cuenta destino segun la vinculada
        transf.libro_origen_id = transf_vinculada.libro_id
        # Obteniendo de paso los gastos vinculados de la vinculada
        transf_vinculada.gasto_x_transferencia.each do |gxt|
          GastoXTransferencia.create(:gasto_id => gxt.gasto_id, :transferencia_id => transf.id)
        end
        # Y arreglamos la parte del financiador segun lo enviado...
        # ... primero borrando las propias de recepcion
        transf.transferencia_x_agente.destroy_all
        # ... y luego incluyendo las del envio
        transf_vinculada.transferencia_x_agente.each do |txf|
          txf.transferencia_id = transf.id
          txf.save
        end
      end
      transf.save
      if transf.errors.empty?
        transf_vinculada.delete if transf_vinculada 
      else
        puts "--------> ERROR!!!: Ha habido problemas al convertir al nuevo formato la transferencia con id: " + transf.id.to_s
        puts "--------> " + transf.errors.inspect
        errores = true
      end
    end

    # Luego, convertimos las transferencias salientes que aun queden
    Transferencia.all(:conditions => {:entrante_saliente => 'saliente'}).each do |transf|
      # Corregimos el tipo "cheque" (eliminado)
      transf.tipo = "retirada" if transf.tipo == "cheque"
      # Corregimos el tipo "devolucion" para salientes
      transf.tipo = "devolucion" if transf.tipo == "adelanto"
      # Ajustamos los datos al nuevo modelo
      transf.fecha_enviado = transf.fecha
      transf.importe_enviado = transf.importe
      transf.libro_origen_id = transf.libro_id
      transf.libro_destino_id = transf.libro_receptor_emisor_id
      transf.save
      unless transf.errors.empty?
        puts "--------> ERROR!!!: Ha habido problemas al convertir al nuevo formato la transferencia con id: " + transf.id.to_s
        puts "--------> " + transf.errors.inspect
        errors =  true
      end
    end
    # Eliminamos los campos que ya no son utiles
    remove_column :transferencia, :importe
    remove_column :transferencia, :fecha
    remove_column :transferencia, :libro_id
    remove_column :transferencia, :moneda_id
    remove_column :transferencia, :libro_receptor_emisor_id
    remove_column :transferencia, :transferencia_id
    remove_column :transferencia, :entrante_saliente
  end

  def self.down
    # Generamos los viejos campos
    add_column :transferencia, :importe, :decimal, :precision => 24, :scale => 2
    add_column :transferencia, :fecha, :date
    add_column :transferencia, :libro_id, :integer
    add_column :transferencia, :moneda_id, :integer
    add_column :transferencia, :libro_receptor_emisor_id, :integer
    add_column :transferencia, :transferencia_id, :integer
    add_column :transferencia, :entrante_saliente, :string
    Transferencia.reset_column_information

    # Recorremos todas las transferencias
    Transferencia.all(:conditions => {:entrante_saliente => nil}).each do |transf|
      # Para transferencias completadas (direccion entrante y saliente)
      if transf.fecha_enviado && transf.fecha_recibido
        # Genera la transferencia saliente vinculada
        transf_vinculada = Transferencia.new(transf.attributes)
        transf_vinculada.fecha = transf.fecha_enviado
        transf_vinculada.importe = transf.importe_enviado
        transf_vinculada.libro_id = transf.libro_origen_id
        transf_vinculada.moneda_id = Libro.find_by_id(transf.libro_origen_id).moneda_id
        transf_vinculada.libro_receptor_emisor_id = transf.libro_destino_id
        transf_vinculada.entrante_saliente = 'saliente'
        transf_vinculada.transferencia_id = transf.id
        transf_vinculada.tasa_cambio = nil
        transf_vinculada.save

        # Vinculamos tambien los financiadores
        transf.transferencia_x_agente.each { |txa| TransferenciaXAgente.create(:transferencia_id => transf_vinculada.id, :agente_id => txa.agente_id, :importe => txa.importe) }

        # Y modifica los datos propios como transferencia entrante
        transf.fecha = transf.fecha_recibido
        transf.importe = transf.importe_cambiado
        transf.libro_id = transf.libro_destino_id
        transf.moneda_id = Libro.find_by_id(transf.libro_destino_id).moneda_id
        transf.libro_receptor_emisor_id = transf.libro_origen_id
        transf.transferencia_id = transf_vinculada.id
        transf.entrante_saliente = 'entrante'
        transf.save
      # Para transferencias salientes
      elsif transf.fecha_enviado
        transf.fecha = transf.fecha_enviado
        transf.importe = transf.importe_enviado
        transf.libro_id = transf.libro_origen_id
        transf.moneda_id = Libro.find_by_id(transf.libro_origen_id).moneda_id
        transf.libro_receptor_emisor_id = transf.libro_destino_id
        transf.entrante_saliente = 'saliente'
        transf.tasa_cambio = nil
        transf.save
      # Para el resto (transf. entrantes)
      elsif transf.fecha_recibido
        transf.fecha = transf.fecha_recibido
        transf.importe = transf.importe_cambiado
        transf.libro_id = transf.libro_destino_id
        transf.moneda_id = Libro.find_by_id(transf.libro_destino_id).moneda_id
        transf.libro_receptor_emisor_id = transf.libro_origen_id
        transf.entrante_saliente = 'entrante'
        transf.save
      end
    end
    # Eliminamos los nuevos campos
    remove_column :transferencia, :fecha_enviado 
    remove_column :transferencia, :importe_enviado
    remove_column :transferencia, :libro_origen_id
    remove_column :transferencia, :fecha_recibido 
    remove_column :transferencia, :importe_recibido
    remove_column :transferencia, :importe_cambiado
    remove_column :transferencia, :libro_destino_id
  end
end
