# This file should contain all the record creation needed to seed the database with its default values.
# The data can then be loaded with the rake db:seed (or created alongside the db with db:setup).
#
# Examples:
#   
#   cities = City.create([{ :name => 'Chicago' }, { :name => 'Copenhagen' }])
#   Major.create(:name => 'Daley', :city => cities.first)

# Incluimos las traducciones
include GetText
bindtextdomain("gor", :path => "locale")
set_locale(ENV['LANG'] || "es_ES")


# Monedas
if Moneda.count == 0
  Moneda.create :nombre => "Euro", :abreviatura => "EUR"
  Moneda.create :nombre => "US Dolar", :abreviatura => "USD"
  Moneda.create :nombre => "Real", :abreviatura => "BRL"
end

# Pais y Area Geografica
if Pais.count == 0
  AreaGeografica.create :nombre => _("Europa")
  AreaGeografica.create :nombre => _("Sudamérica")
  Pais.create :nombre => _("España"), :area_geografica_id => AreaGeografica.find_by_nombre(_("Europa")).id
  Pais.create :nombre => _("Brasil"), :area_geografica_id => AreaGeografica.find_by_nombre(_("Sudamérica")).id
  MonedaXPais.create :pais_id => Pais.find_by_nombre(_("España")).id, :moneda_id => Moneda.find_by_nombre("Euro").id
  MonedaXPais.create :pais_id => Pais.find_by_nombre(_("Brasil")).id, :moneda_id => Moneda.find_by_nombre("Real").id
end

# Provincia.create([{ :nombre => 'Europa', :pais_id => '1' }]) if Provincia.count == 0
# Comunidad.create([{ :nombre => 'Madrid', :provincia_id => '1' }]) if Comunidad.count == 0
# Municipio.create([{ :nombre => 'Madrid', :comunidad_id => '1' }]) if Municipio.count == 0

# Areas de Actuacion

# Sectores de Intervencion

# Sectores Beneficiarios

# Etiquetas

# Socios
if TipoCuotaSocio.count == 0
  TipoCuotaSocio.create :tipo_cuota => _('Mensual'), :meses => 1
  TipoCuotaSocio.create :tipo_cuota => _('Bimensual'), :meses => 2
  TipoCuotaSocio.create :tipo_cuota => _('Trimestral'), :meses => 3
  TipoCuotaSocio.create :tipo_cuota => _('Semestral'), :meses => 6
  TipoCuotaSocio.create :tipo_cuota => _('Anual'), :meses => 12
end

OrigenSocio.create([{ :origen => 'Otro' }]) if OrigenSocio.count == 0

if FormaPagoSocio.count == 0
  FormaPagoSocio.create :forma_pago => _('Efectivo')
  FormaPagoSocio.create :forma_pago => _('Cheque')
  FormaPagoSocio.create :forma_pago => _('Transferencia Bancaria')
  FormaPagoSocio.create :forma_pago => _('Domiciliación Bancaria')
  FormaPagoSocio.create :forma_pago => _('Tarjeta')
  FormaPagoSocio.create :forma_pago => _('Otro')
end 

NaturalezaSocio.create([{ :naturaleza => _('Otro') }]) if NaturalezaSocio.count == 0

# Usuario Administrador
admin = Usuario.find_by_nombre('admin')
if admin.nil? && Usuario.count == 0
  admin = Usuario.create([{ :nombre => 'admin', :contrasena => Digest::SHA1.hexdigest('admin'), :correoe => 'gong@ejemplo.org', :administracion => true, :proyectos=> true, :agentes => true, :cuadromando => true, :socios => true, :documentos => true}])
end

# Estados de los Proyectos (WorkFlow)
if DefinicionEstado.count == 0
  contacto = DefinicionEstado.create :nombre => _("Contacto"), :orden => 0, :primer_estado => true
  identificacion = DefinicionEstado.create :nombre => _("Identificación"), :orden => 1, :estado_padre_id => contacto.id, :primer_estado => false, :formulacion => true
  formulacion = DefinicionEstado.create :nombre => _("Formulación"), :orden => 2, :estado_padre_id => identificacion.id, :primer_estado => false, :formulacion => true
  ejecucion = DefinicionEstado.create :nombre => _("Ejecución"), :orden => 3, :estado_padre_id => formulacion.id, :primer_estado => false, :aprobado => true
  reformulacion = DefinicionEstado.create :nombre => _("Reformulación"), :orden => 3, :estado_padre_id => formulacion.id, :primer_estado => false, :formulacion => true, :aprobado => true
  DefinicionEstado.create :nombre => _("Cerrado"), :orden => 5, :estado_padre_id => reformulacion.id, :primer_estado => false, :cerrado => true
end

# Tipos de Tarea
if TipoTarea.count == 0
  TipoTarea.create (:nombre =>  _("Datos proyecto"), :tipo_proyecto => true)
  TipoTarea.create (:nombre =>  _("Documentación"), :tipo_proyecto => true)
  TipoTarea.create (:nombre =>  _("Presupuesto"), :tipo_proyecto => true)
  TipoTarea.create (:nombre =>  _("Gasto"), :tipo_proyecto => true)
  TipoTarea.create (:nombre =>  _("Matriz"), :tipo_proyecto => true)
end

# Estados de las Tareas
if EstadoTarea.count == 0
  EstadoTarea.create( :nombre => _("Cerrada"), :descripcion => _("Tarea que ya está finalizada"), :activo => false)
  EstadoTarea.create( :nombre => _("En curso"), :descripcion => _("Tarea en la cual se trabaja actualmente"), :activo => true)
  EstadoTarea.create( :nombre => _("En espera"), :descripcion => _("Tarea que todavía no ha comenzado"), :activo => true)
end

# Agente
if Agente.count == 0
  # Forzamos textos para traducir
  a = _("OTRAS FINANCIACIONES PÚBLICAS")
  b = _("OTRAS FINANCIACIONES PRIVADAS")
  Agente.create :nombre => "AECID", :financiador => true
  Agente.create :nombre => "ONG-INT", :financiador => true, :implementador => true, :pais_id => Pais.find_by_nombre(_("España")).id, :moneda_id => Moneda.find_by_nombre("Euro").id
  Agente.create :nombre => "OTRAS FINANCIACIONES PÚBLICAS", :financiador => true
  Agente.create :nombre => "OTRAS FINANCIACIONES PRIVADAS", :financiador => true
end

# Espacios para Gestión Documental
if Espacio.count == 0
  # Forzamos textos para traducir
  a = _("Proyectos")
  b = _("Agentes")
  # Espacios para Proyectos
  Espacio.create :nombre => "Proyectos", :descripcion => _("Espacio raiz de los espacios de proyecto. Este espacio no se puede modificar ni eliminar.")
  #Espacio.create :nombre => "Gastos", :definicion_espacio_proyecto => true, :gasto => true
  #Espacio.create :nombre => "Fuentes de verificación", :definicion_espacio_proyecto => true, :fuente_verificacion => true
  #Espacio.create :nombre => "Transferencias", :definicion_espacio_proyecto => true, :transferencia => true
  # Espacios para agentes
  Espacio.create :nombre => "Agentes", :descripcion => _("Espacio base de los espacios de agente")
  #Espacio.create :nombre => "Transferencias", :definicion_espacio_agente => true, :transferencia => true
  #Espacio.create :nombre => "Gastos", :definicion_espacio_agente => true, :gasto => true
end

# Proyectos de Ejemplo
if Proyecto.count == 0
  proyecto = Proyecto.create :nombre => "2012-GONG", :titulo => _("Proyecto para la implantación en nuestra ONG de GONG"), :moneda_id => Moneda.find_by_nombre("Euro").id, :agente_id => Agente.find_by_nombre("AECID").id, :fecha_convocatoria => Date.today, :pais_principal_id => Pais.find_by_nombre(_("España"))

  # Si existe el administrador le asigna todo a el
  if admin && proyecto
    Estado.create :proyecto_id => proyecto.id, :fecha_inicio => Date.today, :usuario_id => admin, :definicion_estado_id => DefinicionEstado.find_by_primer_estado(true), :estado_actual => true
    UsuarioXProyecto.create :usuario_id => admin.id, :proyecto_id => proyecto.id, :rol => "coordinador"

    # Tareas para el Usuario Admin
    Tarea.create(:titulo => _("Dar de alta etapas del proyecto"), :proyecto_id => proyecto.id, :usuario_asignado_id => admin.id, :fecha_inicio => Date.today, :tipo_tarea_id =>  TipoTarea.find_by_nombre(_("Datos proyecto")).id, :estado_tarea_id => EstadoTarea.find_by_nombre(_("En espera")).id)
    Tarea.create(:titulo => _("Cambiar estado proyecto"), :proyecto_id => proyecto.id, :usuario_asignado_id => admin.id, :fecha_inicio => Date.today, :tipo_tarea_id =>  TipoTarea.find_by_nombre(_("Datos proyecto")).id, :estado_tarea_id => EstadoTarea.find_by_nombre(_("En espera")).id)
    Tarea.create(:titulo => _("Dar de alta relaciones proyecto"), :descripcion => _("Se necesita dar de alta las entidades en administracion"), :proyecto_id => proyecto.id, :usuario_asignado_id => admin.id, :fecha_inicio => Date.today, :tipo_tarea_id =>  TipoTarea.find_by_nombre(_("Datos proyecto")).id, :estado_tarea_id => EstadoTarea.find_by_nombre(_("En espera")).id)
    Tarea.create(:titulo => _("Dar de alta el presupuesto"), :proyecto_id => proyecto.id, :usuario_asignado_id => admin.id, :fecha_inicio => Date.today, :tipo_tarea_id =>  TipoTarea.find_by_nombre(_("Presupuesto")).id, :estado_tarea_id => EstadoTarea.find_by_nombre(_("En espera")).id)
    Tarea.create(:titulo => _("Dar de alta la matriz"), :proyecto_id => proyecto.id, :usuario_asignado_id => admin.id, :fecha_inicio => Date.today, :tipo_tarea_id =>  TipoTarea.find_by_nombre(_("Matriz")).id, :estado_tarea_id => EstadoTarea.find_by_nombre(_("En espera")).id)
  end
end

